@@ -73,7 +73,8 @@ module Exploit::Remote::HttpServer
   end
 
   def print_prefix
-    if cli && !(stance == Msf::Exploit::Stance::Aggressive || stance.include?(Msf::Exploit::Stance::Aggressive))
+    if cli && self.respond_to?(:stance) &&
+        !(stance == Msf::Exploit::Stance::Aggressive || stance.include?(Msf::Exploit::Stance::Aggressive))
       super + "#{cli.peerhost.ljust(16)} #{self.shortname} - "
     else
       super
@@ -325,7 +326,7 @@ module Exploit::Remote::HttpServer
       when /ppc/
         fp[:arch] = ARCH_PPC
       when /x64|x86_64/
-        fp[:arch] = ARCH_X86_64
+        fp[:arch] = ARCH_X64
       when /i.86|wow64/
         # WOW64 means "Windows on Windows64" and is present
         # in the useragent of 32-bit IE running on 64-bit
