@@ -71,6 +71,7 @@ module Msf
           "nessus_index" => "Manually generates a search index for exploits",
           "nessus_template_list" => "List all the templates on the server",
           "nessus_db_scan" => "Create a scan of all IP addresses in db_hosts",
+          "nessus_db_scan_workspace" => "Create a scan of all IP addresses in db_hosts for a given workspace",
           "nessus_db_import" => "Import Nessus scan to the Metasploit connected database",
           "nessus_save" => "Save credentials of the logged in user to nessus.yml",
           "nessus_folder_list" => "List folders configured on the Nessus server",
@@ -256,6 +257,7 @@ module Msf
         tbl << [ "Nessus Database Commands", "" ]
         tbl << [ "-----------------", "-----------------" ]
         tbl << [ "nessus_db_scan", "Create a scan of all IP addresses in db_hosts" ]
+        tbl << [ "nessus_db_scan_workspace", "Create a scan of all IP addresses in db_hosts for a given workspace" ]
         tbl << [ "nessus_db_import", "Import Nessus scan to the Metasploit connected database" ]
         tbl << [ "", ""]
         tbl << [ "Reports Commands", "" ]
@@ -966,13 +968,14 @@ module Msf
         end
         if valid_policy(uuid)
           print_status("Creating scan from policy number #{uuid}, called #{scan_name} - #{description} and scanning #{targets}")
-          et=Hash.new
-          et['enabled']=false
-          et['launch']='ONETIME'
-          et['name']=scan_name
-          et['text_targets']=targets
-          et['description']=description
-          et['launch_now']=false
+          et = {
+            'enabled'      => false,
+            'launch'       => 'ONETIME',
+            'name'         => scan_name,
+            'text_targets' => targets,
+            'description'  => description,
+            'launch_now'   => false
+          }
           scan = @n.scan_create(uuid, et)
           tbl = Rex::Text::Table.new(
             'Columns' => [
@@ -1075,13 +1078,14 @@ module Msf
         end
         targets.chop!
         print_status("Creating scan from policy #{policy_id}, called \"#{name}\" and scanning all hosts in all the workspaces")
-        et=Hash.new
-        et['enabled']=false
-        et['launch']='ONETIME'
-        et['name']=name
-        et['text_targets']=targets
-        et['description']=desc
-        et['launch_now']=true
+        et = {
+          'enabled'      => false,
+          'launch'       => 'ONETIME',
+          'name'         => name,
+          'text_targets' => targets,
+          'description'  => desc,
+          'launch_now'   => true
+        }
         scan = @n.scan_create(policy_id, et)
         if !scan["error"]
           scan = scan["scan"]
@@ -1091,6 +1095,67 @@ module Msf
         end
       end
 
+      def cmd_nessus_db_scan_workspace(*args)
+        if args[0] == "-h"
+          print_status("nessus_db_scan_workspace <policy ID> <scan name> <scan description> <workspace>")
+          print_status("Creates a scan based on all the hosts listed in db_hosts for a given workspace.")
+          print_status("Use nessus_policy_list to list all available policies with their corresponding policy IDs")
+          return
+        end
+        if !nessus_verify_db
+          return
+        end
+        if !nessus_verify_token
+          return
+        end
+        case args.length
+        when 4
+          policy_id = args[0]
+          name = args[1]
+          desc = args[2]
+          new_workspace = framework.db.find_workspace(args[3])
+        else
+          print_status("Usage: ")
+          print_status("nessus_db_scan_workspace <policy ID> <scan name> <scan description> <workspace>")
+          print_status("Use nessus_policy_list to list all available policies with their corresponding policy IDs")
+          return
+        end
+        if !valid_policy(policy_id)
+          print_error("That policy does not exist.")
+          return
+        end
+        if new_workspace.nil?
+          print_error("That workspace does not exist.")
+          return
+        end
+        framework.db.workspace = new_workspace
+        print_status("Switched workspace: #{framework.db.workspace.name}")
+        targets = ""
+        framework.db.hosts.each do |host|
+          targets << host.address
+          targets << ","
+        print_status("Targets: #{targets}")
+        end
+        targets.chop!
+        print_status("Creating scan from policy #{policy_id}, called \"#{name}\" and scanning all hosts in #{framework.db.workspace.name}")
+        et = {
+          'enabled'      => false,
+          'launch'       => 'ONETIME',
+          'name'         => name,
+          'text_targets' => targets,
+          'description'  => desc,
+          'launch_now'   => false
+        }
+        scan = @n.scan_create(policy_id, et)
+        if !scan["error"]
+          scan = scan["scan"]
+          print_status("Scan ID #{scan['id']} successfully created")
+          print_status("Run nessus_scan_launch #{scan['id']} to launch the scan")
+        else
+          print_error(JSON.pretty_generate(scan))
+        end
+      end
+
       def cmd_nessus_db_import(*args)
         if args[0] == "-h"
           print_status("nessus_db_import <scan ID>")
@@ -1119,8 +1184,15 @@ module Msf
             file_id = export["file"]
             print_good("The export file ID for scan ID #{scan_id} is #{file_id}")
             print_status("Checking export status...")
-            status = @n.scan_export_status(scan_id, file_id)
-            if status == "ready"
+            begin
+              status = @n.scan_export_status(scan_id, file_id)
+              print_status("Export status: " + status["status"])
+              if status["status"]=="ready"
+                break
+              end     
+              sleep(1)
+            end while (status["status"]=="loading")
+            if status["status"] == "ready"
               print_status("The status of scan ID #{scan_id} export is ready")
               select(nil, nil, nil, 5)
               report = @n.report_download(scan_id, file_id)
@@ -1394,9 +1466,16 @@ module Msf
           if export["file"]
             file_id = export["file"]
             print_good("The export file ID for scan ID #{scan_id} is #{file_id}")
-            print_status("Checking export status...")
-            status = @n.scan_export_status(scan_id, file_id)
-            if status == "ready"
+            print_status("Checking export status...")            
+            begin
+              status = @n.scan_export_status(scan_id, file_id)
+              print_status("Export status: " + status["status"])
+              if status["status"]=="ready"
+                break
+              end     
+              sleep(1)
+            end while (status["status"]=="loading")
+            if status["status"] == "ready"
               print_good("The status of scan ID #{scan_id} export is ready")
             else
               print_error("There was some problem in exporting the scan. The error message is #{status}")
@@ -1422,8 +1501,15 @@ module Msf
         when 2
           scan_id = args[0]
           file_id = args[1]
-          status = @n.scan_export_status(scan_id, file_id)
-          if status == "ready"
+          begin
+            status = @n.scan_export_status(scan_id, file_id)
+            print_status("Export status: " + status["status"])
+            if status["status"]=="ready"
+              break
+            end     
+            sleep(1)
+          end while (status["status"]=="loading")
+          if status["status"] == "ready"
             print_status("The status of scan ID #{scan_id} export is ready")
           else
             print_error("There was some problem in exporting the scan. The error message is #{status}")
